#ifndef __ARITHMETIC_H__
#define __ARITHMETIC_H__

template<class T>
T _max(T lhs, T rhs)
{
	return (lhs > rhs) ? lhs : rhs;
}

template<class T>
T _min(T lhs, T rhs)
{
	return (lhs < rhs) ? lhs : rhs;
}

template<int Mod>
class IntMod
{
public:
	IntMod(int value=0)
		: m_val(value % Mod)
	{
	}

	IntMod<Mod> operator + (const IntMod<Mod>& rhs) const
	{
		return IntMod<Mod>(m_val + rhs.m_val);
	}

	IntMod<Mod> operator * (const IntMod<Mod>& rhs) const
	{
		return IntMod<Mod>(m_val * rhs.m_val);
	}

	bool operator == (const IntMod<Mod>& rhs) const
	{
		return m_val == rhs.m_val;
	}

	bool operator != (const IntMod<Mod>& rhs) const
	{
		return !(*this == rhs);
	}

	int GetVal() const {return m_val;}

	IntMod<Mod>	Inverse() const
	{
		return (Mod - m_val) % Mod;
	}

	void Invert()
	{
		m_val = (Mod - m_val) % Mod;
	}

private:
	int m_val;
};

template<int n>
std::ostream& operator << (std::ostream& stream, const IntMod<n>& num)
{
	return stream << num.GetVal();
}


template<class A, class B, class C, class D, class E, class F>
class SixTuplet
{
public:
	SixTuplet()
		: first(A(0)), second(B(0))
		, third(C(0)), fourth(D(0))
		, fifth(E(0)), sixth(F(0))
	{
	}

	SixTuplet(A a, B b, C c, D d, E e, F f)
		: first(a), second(b), third(c), fourth(d), fifth(e), sixth(f)
	{
	}

	typedef SixTuplet<A, B, C, D, E, F>	MyT;
	typedef A	First_t;
	typedef B	Second_t;
	typedef C	Third_t;
	typedef D	Fourth_t;
	typedef E	Fifth_t;
	typedef F	Sixth_t;

	MyT operator + (const MyT& rhs) const
	{
		return MyT(first + rhs.first, second + rhs.second, third + rhs.third, fourth + rhs.fourth, fifth + rhs.fifth, sixth + rhs.sixth);
	}

	bool operator == (const MyT& rhs) const
	{
		return (first == rhs.first && second == rhs.second && third == rhs.third && fourth == rhs.fourth && fifth == rhs.fifth && sixth == rhs.sixth);
	}

	bool operator != (const MyT& rhs) const
	{
		return !(*this == rhs);
	}
	
	SixTuplet Inverse() const
	{
		return SixTuplet(first.Inverse(), second.Inverse(), third.Inverse(), fourth.Inverse(), fifth.Inverse(), sixth.Inverse());
	}

private:
	First_t		first;
	Second_t	second;
	Third_t		third;
	Fourth_t	fourth;
	Fifth_t		fifth;
	Sixth_t		sixth;
};


template<class A, class B, class C, class D, class E, class F>
inline std::ostream& operator << (std::ostream& stream, const SixTuplet<A,B,C,D,E,F>& elem)
{
	stream << "(" << first.GetVal() << "," << second.GetVal() << "," << third.GetVal() << "," << fourth.GetVal() << "," << fifth.GetVal() << "," << sixth.GetVal() << ")";
	return stream;
}

#endif