#ifndef __SEQUENCEBLOCK_H__
#define __SEQUENCEBLOCK_H__

#include "Typedef.h"

class FiniteAbelianGroup;

class SequenceBlock
{
public:
	typedef Sequence_t::const_iterator	SequenceConstIter_t;
	typedef Sequence_t::iterator		SequenceIter_t;
	typedef std::vector<bool>			List_t;
	typedef List_t::iterator			ListIter_t;
	typedef List_t::const_iterator		ListConstIter_t;

	SequenceBlock(FiniteAbelianGroup& g, const Sequence_t& rSequence, const Set_t& rSubsums);
	SequenceBlock(const SequenceBlock& rhs);

	SequenceIter_t		sequence_begin()			{return m_Sequence.begin();}
	SequenceConstIter_t	sequence_begin()	const	{return m_Sequence.begin();}
	Sequence_t::reference	sequence_front()		{return m_Sequence.front();}

	SequenceIter_t		sequence_end()				{return m_Sequence.end();}
	SequenceConstIter_t	sequence_end()		const	{return m_Sequence.end();}
	Sequence_t::reference	sequence_back()			{return m_Sequence.back();}

	int					sequence_size()		const	{return m_Sequence.size();}

	ListConstIter_t		L1_begin()	const	{return m_Subsums.begin();}
	ListIter_t			L1_begin()			{return m_Subsums.begin();}
	ListConstIter_t		L1_end()	const	{return m_Subsums.end();}
	ListIter_t			L1_end()			{return m_Subsums.end();}
	int					L1_size()	const	{return m_Subsums.size();}

	ListConstIter_t		L2_begin()	const	{return m_Complements.begin();}
	ListIter_t			L2_begin()			{return m_Complements.begin();}
	ListConstIter_t		L2_end()	const	{return m_Complements.end();}
	ListIter_t			L2_end()			{return m_Complements.end();}
	int					L2_size()	const	{return m_Complements.size();}

	bool operator < (const SequenceBlock& rhs) const;

private:
	FiniteAbelianGroup&	m_Group;

	Sequence_t	m_Sequence;

	List_t		m_Subsums;
	List_t		m_Complements;
};


typedef std::set<SequenceBlock*, val_less<SequenceBlock*> >				SequenceBlockList_t;
typedef std::map<int, SequenceBlockList_t*>	SequenceBlockLookup_t;


#endif