#include "stdafx.h"
#include "SequenceClasses.h"

OrderClass::OrderClass(const std::vector<int>& rDivisorList)
{
	std::vector<int>::const_iterator iter = rDivisorList.begin();
	std::vector<int>::const_iterator iterEnd = rDivisorList.end();

	for (; iter != iterEnd; ++iter)
	{
		m_Lookup[*iter] = 0;
	}
}

int OrderClass::GetOrderValue(int nOrder) const
{
	Lookup_t::const_iterator iter = m_Lookup.find(nOrder);
	if (iter == m_Lookup.end())
		throw;

	return (*iter).second;
}

void OrderClass::SetOrderValue(int nOrder, int nVal)
{
	Lookup_t::const_iterator iter = m_Lookup.find(nOrder);
	if (iter == m_Lookup.end())
		throw;

	m_Lookup[nOrder] = nVal;
}

bool OrderClass::operator < (const OrderClass& rhs) const
{
	if (length() != rhs.length())
		throw;

	return m_Lookup < rhs.m_Lookup;
}


std::ostream& OrderClass::ToStream(std::ostream& rStream) const
{
	rStream << "(";

	Lookup_t::const_iterator iter = m_Lookup.begin();
	Lookup_t::const_iterator iterEnd = m_Lookup.end();

	std::vector<std::pair<int, int> >	nonZeroLookup;
	for (; iter != iterEnd; ++iter)
	{
		const Lookup_t::value_type& rPair = *iter;

		if (rPair.second != 0)
			nonZeroLookup.push_back(rPair);
	}

	std::vector<std::pair<int, int> >::const_iterator iter2 = nonZeroLookup.begin();
	std::vector<std::pair<int, int> >::const_iterator iter2End = nonZeroLookup.end();
	while (iter2 != iter2End)
	{
		const std::pair<int, int>& rPair = *iter2;

		rStream << rPair.first << ":" << rPair.second;

		if (++iter2 != iter2End)
			rStream << ", ";
	}


	rStream << ")";
	return rStream;
}

int& OrderClass::operator[](int nOrder)
{
	return m_Lookup[nOrder];
}

int OrderClass::operator[](int nOrder) const
{
	return const_cast<OrderClass*>(this)->m_Lookup[nOrder];
}


std::ostream& operator << (std::ostream& stream, const OrderClass& rhs)
{
	return rhs.ToStream(stream);
}

MultiplicityClass::MultiplicityClass(const CodedSequence_t& rSequence)
{
	CodedSequence_t::const_iterator iter = rSequence.begin();
	CodedSequence_t::const_iterator iterEnd = rSequence.end();

	for (; iter != iterEnd; ++iter)
	{
		const CodedSequence_t::value_type& rPair = *iter;

		m_Lookup.insert(rPair.second);
	}
}

std::ostream& MultiplicityClass::ToStream(std::ostream& rStream) const
{
	char cLetter = 'a';

	Lookup_t::const_iterator iter = m_Lookup.begin();
	Lookup_t::const_iterator iterEnd = m_Lookup.end();
	for (;iter != iterEnd; ++iter, ++cLetter)
	{
		int value = *iter;

		rStream << cLetter;
		if (value > 1)
			rStream << "^" << value;
	}

	return rStream;
}

bool MultiplicityClass::operator < (const MultiplicityClass& rhs) const
{
	int n = std::accumulate(m_Lookup.begin(), m_Lookup.end(), 0);
	int n2 = std::accumulate(rhs.m_Lookup.begin(), rhs.m_Lookup.end(), 0);

	if (n < n2)
		return true;

	if (n2 < n)
		return false;

	return m_Lookup < rhs.m_Lookup;
}


std::ostream& operator << (std::ostream& stream, const MultiplicityClass& rhs)
{
	return rhs.ToStream(stream);
}
